#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <stddef.h>
#include <tinycrypt/aes.h>
#include <tinycrypt/constants.h>
#include <zephyr/sys/printk.h>
#include "aes_comm.h"
#include "error.h"

int tcAes128Decrypt(const uint8_t* pCiphertext, uint8_t* pPlaintext) {
    const uint8_t* aes128Key = aes128GetKey();
    struct tc_aes_key_sched_struct s;

    if (tc_aes128_set_decrypt_key(&s, aes128Key) != TC_CRYPTO_SUCCESS) {
        printk("AES128 set decryption key failed.\n");
        return ERROR_AES_DECRYPT_FAIL;
    }

    if (tc_aes_decrypt(pPlaintext, pCiphertext, &s) != TC_CRYPTO_SUCCESS) {
        printk("AES128 decryption failed.\n");
        return ERROR_AES_DECRYPT_FAIL;
    }
    return 0;
}

int tcAes128Encrypt(const uint8_t* pPlaintext, uint8_t* pCiphertext) {
    const uint8_t* aes128Key = aes128GetKey();
    struct tc_aes_key_sched_struct s;

    if (tc_aes128_set_encrypt_key(&s, aes128Key)!= TC_CRYPTO_SUCCESS) {
        printk("AES128 set encryption key failed.\n");
        return ERROR_AES_ENCRYPT_FAIL;
    }

    if (tc_aes_encrypt(pCiphertext, pPlaintext, &s) != TC_CRYPTO_SUCCESS) {
        printk("AES128 encryption failed.\n");
        return ERROR_AES_ENCRYPT_FAIL;
    }

    return 0;
}

int tcAes128Test(void) {
    uint8_t ciphertext[AES128_KEY_BYTES];
    const uint8_t plaintext[AES128_KEY_BYTES] = {
        0x32, 0x43, 0xf6, 0xa8, 0x88, 0x5a, 0x30, 0x8d,
        0x31, 0x31, 0x98, 0xa2, 0xe0, 0x37, 0x07, 0x34
    };
    uint8_t plaintext2[AES128_KEY_BYTES];

    tcAes128Encrypt(plaintext, ciphertext);
    tcAes128Decrypt(ciphertext, plaintext2);

    for (int i=0; i<AES128_KEY_BYTES; i++) {
        if (plaintext[i] != plaintext2[i]) {
            printk("AES encryption/decryption test failed\n");
            return -1;
        }
    }

    printk("AES encryption/decryption test passed\n");

    return 0;
}
